
  - name: {{ catalog['name'] }} {{ catalog['os_version'] }}
    format: qcow2
    login: almalinux
    min_disk: 8
    min_ram: 512
    status: active
    visibility: public
    multi: true
    meta:
      architecture: x86_64
      hypervisor_type: qemu
      hw_disk_bus: scsi
      hw_rng_model: virtio
      hw_scsi_model: virtio-scsi
      hw_qemu_guest_agent: yes
      hw_watchdog_action: reset
      replace_frequency: monthly
      hotfix_hours: 0
      uuid_validity: last-3
      provided_until: none
      image_source: foo
      os_distro: almalinux
      os_version: '{{ catalog['os_version'] }}'
    tags: []
    latest_checksum_url: {{ metadata['baseURL'] }}{{ metadata['releasepath'] }}/{{ metadata['checksumname'] }}
    latest_url: {{ metadata['baseURL'] }}{{ metadata['releasepath'] }}/{{ metadata['imagename'] }}.{{ metadata['extension'] }}
    versions:{% for release_version in catalog['versions'] %}
      - version: '{{ release_version }}'
        url: {{ catalog['versions'][release_version]['url'] }}
        checksum: {{ catalog['versions'][release_version]['checksum'] }}
        build_date: {{ catalog['versions'][release_version]['release_date'] }}
        image_source: {{ catalog['versions'][release_version]['url'] }}
        image_description: https://wiki.almalinux.org/release-notes/
{%- endfor %}
