INSERT INTO image_catalog (name, release_date, version, distribution_name, distribution_release, url, checksum) VALUES ('Ubuntu 20.04 LTS', '2022-06-27', '20220627.1', 'Ubuntu', '20.04', 'https://cloud-images.ubuntu.com/releases/focal/release-20220627.1/ubuntu-20.04-server-cloudimg-amd64.img', 'sha256:cd55216608b2d5238a7d0a106d91f5f91f1daed7d53ef9b8c11605f1a8b1d814');
INSERT INTO image_catalog (name, release_date, version, distribution_name, distribution_release, url, checksum) VALUES ('Ubuntu 20.04 LTS', '2022-07-06', '20220706', 'Ubuntu', '20.04', 'https://cloud-images.ubuntu.com/releases/focal/release-20220706/ubuntu-20.04-server-cloudimg-amd64.img', 'sha256:7bd0d8d1426013c659411abefde41f4cbbbba5f2c0fdb5b8ea7a5c955998fe3c');
